# -*- coding: utf-8 -*-

"""
MEDIA:
    Diario El Independiente, Baja California Sur

USAGE:
    $ cd elIndependiente/
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to the oldest. It's necessary to use the parse_date_files.py file
    for the news contained in noticias.json being splitted into files by date.
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    ------------------------------------------------------------------------------------------------------------
    ## Get all the news from the most current to a specific date. ##
    $ scrapy crawl noticias --nolog -s filename=2018-08-10.json -a year=2018 -a month=8 -a day=10
"""

import scrapy, re, json
from datetime import datetime, date
from diarioIndependiente.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)



class ImportantFlowData(scrapy.Item):
    """
    Useful data for the flow of the implementation
    """
    TO_NEXT_PAGE = scrapy.Field()
    IS_LAST_LINK = scrapy.Field()
    URL          = scrapy.Field()



class QuotesSpider(scrapy.Spider):
    """
    Basic Scrapy Spider class
    """
    name = "noticias"


    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stop_date = date(int(year), int(month), int(day))
            base_url       = "https://www.diarioelindependiente.mx/" + year + "/" + month + "/"

        else:
            self.stop_date = None
            section_list   = ["la-paz", "los-cabos", "policiaca", "deportes", "cultura", "nacional",
                              "internacional", "opinion", "espectaculos", "tecnologia"]
            
            base_url = "https://www.diarioelindependiente.mx/"        

        
        if self.stop_date is None:
            for s in section_list:
                yield scrapy.Request(url=base_url + s, callback=self.parse)
        
        else:
            flow_info                 = ImportantFlowData()
            flow_info['TO_NEXT_PAGE'] = False
            
            request              = scrapy.Request(url=base_url, callback=self.parse_with_stop_date)
            request.meta['item'] = flow_info
            yield request



    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.css('div.paginacion').xpath('./ul/li/a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-2]
            pages      = int(pagination[pagination.rfind('=') + 1:])

            for page in xrange(1, pages):
                yield scrapy.Request(url=response.url + "?page=" + str(page + 1), callback=self.parse_page)



    def parse_page(self, response):
        link_list = response.xpath('//div[@id="colNoticias"]').css('article.card__article').xpath('./h2/a/@href').extract()

        for link in link_list:
            yield scrapy.Request(url=link, callback=self.parse_item)



    def parse_with_stop_date(self, response):
        flow_info    = response.meta['item']
        TO_NEXT_PAGE = flow_info['TO_NEXT_PAGE']

        if not TO_NEXT_PAGE:
            link_list = response.xpath('//div[@id="colNoticias"]').css('article.card__article').xpath('./h2/a/@href').extract()

            for link in link_list:
                flow_info        = ImportantFlowData()
                flow_info['URL'] = response.url

                if link == link_list[-1] : flow_info['IS_LAST_LINK'] = True
                else                     : flow_info['IS_LAST_LINK'] = False

                request              = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request

        else:
            next_page = response.css('div.paginacion').xpath('./ul/li/a[@rel="next"]/@href').extract_first()

            if next_page is not None:
                flow_info['TO_NEXT_PAGE'] = False

                request              = scrapy.Request(url=next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request



    def parse_item(self, response):
        item      = NoticiasItem()
        text      = ''

        news_date = response.xpath('//meta[@name="date"]/@content').extract_first()

        try:
            topic = response.xpath('//span[@class="badge"]').extract_first()
        except:
            topic = None

        for p in response.css('div.cuerpo_noticia').css('p').extract():
                text += remove_tags(p) + "\n"

        item['date']  = datetime.strptime(news_date, '%Y-%m-%d').isoformat("T")
        item['title'] = remove_tags(response.css('h1.colorRojo').extract_first())
        item['topic'] = remove_tags(topic)
        item['text']  = text.strip()
        item['url']   = response.url

        yield item



    def parse_item_with_stop_date(self, response):
        news_date = response.xpath('//meta[@name="date"]/@content').extract_first()
        news_date = datetime.strptime(news_date, '%Y-%m-%d').date()

        if news_date >= self.stop_date:
            flow_info = response.meta['item']
            item      = NoticiasItem()
            text      = ''

            try:
                topic = response.xpath('//span[@class="badge"]').extract_first()
            except:
                topic = None

            for p in response.css('div.cuerpo_noticia').css('p').extract():
                text += remove_tags(p) + "\n"

            item['date']  = datetime.strptime(news_date.isoformat(), '%Y-%m-%d').isoformat("T")
            item['title'] = remove_tags(response.css('h1.colorRojo').extract_first())
            item['topic'] = remove_tags(topic)
            item['text']  = text.strip()
            item['url']   = response.url

            yield item

            if flow_info['IS_LAST_LINK']:
                flow_info['TO_NEXT_PAGE'] = True
                
                request              = scrapy.Request(url=flow_info['URL'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                
                yield request
